@@ -22,7 +22,7 @@ import six
 from law.config import Config
 from law.job.base import BaseJobManager, BaseJobFileFactory
 from law.target.file import get_scheme
-from law.util import interruptable_popen, make_list, quote_cmd
+from law.util import interruptable_popen, make_list, make_unique, quote_cmd
 
 
 logger = logging.getLogger(__name__)
@@ -346,7 +346,7 @@ class ARCJobFileFactory(BaseJobFileFactory):
         if c.output_uri and "output_uri" not in render_variables:
             render_variables["output_uri"] = c.output_uri
 
-        # linearize render_variables
+        # linearize render variables
         render_variables = self.linearize_render_variables(render_variables)
 
         # prepare the job file
@@ -407,6 +407,11 @@ class ARCJobFileFactory(BaseJobFileFactory):
             c.stdout = c.stdout and self.postfix_file(c.stdout, postfix)
             c.stderr = c.stderr and self.postfix_file(c.stderr, postfix)
 
+        # custom log file
+        if c.custom_log_file:
+            c.output_files.append(prepare_output(c.custom_log_file))
+            c.custom_log_file = self.postfix_file(c.custom_log_file, postfix)
+
         # job file content
         content = []
         content.append(("executable", c.executable))
@@ -415,9 +420,9 @@ class ARCJobFileFactory(BaseJobFileFactory):
         if c.job_name:
             content.append(("jobName", c.job_name))
         if c.input_files:
-            content.append(("inputFiles", c.input_files))
+            content.append(("inputFiles", make_unique(c.input_files)))
         if c.output_files:
-            content.append(("outputFiles", c.output_files))
+            content.append(("outputFiles", make_unique(c.output_files)))
         if c.log:
             content.append(("gmlog", c.log))
         if c.stdout:
@@ -438,7 +443,7 @@ class ARCJobFileFactory(BaseJobFileFactory):
 
         logger.debug("created glite job file at '{}'".format(job_file))
 
-        return job_file
+        return job_file, c
 
     @classmethod
     def create_line(cls, key, value):
