@@ -6,8 +6,9 @@ IPython related law functionality.
 """
 
 
-__all__ = ["register_magics"]
+__all__ = ["Task", "register_magics"]
 
 
 # provisioning imports
+from law.contrib.ipython.task import Task
 from law.contrib.ipython.magic import register_magics
