@@ -21,7 +21,7 @@ from abc import ABCMeta, abstractmethod
 import six
 
 from law.config import Config
-from law.util import colored, make_list, iter_chunks, flatten, quote_cmd
+from law.util import colored, make_list, iter_chunks, flatten
 
 
 def get_async_result_silent(result, timeout=None):
@@ -546,7 +546,7 @@ class BaseJobFileFactory(object):
        is *None*.
     """
 
-    config_attrs = ["dir"]
+    config_attrs = ["dir", "custom_log_file"]
 
     render_key_cre = re.compile(r"\{\{(\w+)\}\}")
 
@@ -570,7 +570,7 @@ class BaseJobFileFactory(object):
         def __contains__(self, attr):
             return attr in self.__dict__
 
-    def __init__(self, dir=None, mkdtemp=None, cleanup=None):
+    def __init__(self, dir=None, custom_log_file=None, mkdtemp=None, cleanup=None):
         super(BaseJobFileFactory, self).__init__()
 
         cfg = Config.instance()
@@ -599,6 +599,9 @@ class BaseJobFileFactory(object):
         if mkdtemp:
             self.dir = tempfile.mkdtemp(dir=self.dir)
 
+        # store the custom log file
+        self.custom_log_file = custom_log_file
+
     def __del__(self):
         self.cleanup_dir(force=False)
 
@@ -709,9 +712,9 @@ class BaseJobFileFactory(object):
             return cls.postfix_file(m.group(1), postfix)
 
         for key, value in six.iteritems(render_variables):
-            # value might contain paths that should be postfixed, denoted by "postfix:..."
+            # value might contain paths to be postfixed, denoted by "__law_job_postfix__:..."
             if postfix:
-                value = re.sub(r"postfix:([^\s]+)", postfix_fn, value)
+                value = re.sub(r"\_\_law\_job\_postfix\_\_:([^\s]+)", postfix_fn, value)
             content = cls.render_string(content, key, value)
 
         # finally, replace all non-rendered keys with empty strings
@@ -841,6 +844,14 @@ class JobArguments(object):
         """
         return "yes" if b else "no"
 
+    @classmethod
+    def encode_string(cls, s):
+        """
+        Encodes a string *s* via base64 encoding.
+        """
+        encoded = base64.b64encode(six.b(s or "-"))
+        return encoded.decode("utf-8") if six.PY3 else encoded
+
     @classmethod
     def encode_list(cls, l):
         """
@@ -849,15 +860,6 @@ class JobArguments(object):
         encoded = base64.b64encode(six.b(" ".join(str(v) for v in l) or "-"))
         return encoded.decode("utf-8") if six.PY3 else encoded
 
-    @classmethod
-    def encode_params(cls, params):
-        """
-        Encodes a list of command line parameters *params* into a string via
-        :py:func:`law.util.quote_cmd` followed by base64 encoding.
-        """
-        encoded = base64.b64encode(six.b(quote_cmd(params) or "-"))
-        return encoded.decode("utf-8") if six.PY3 else encoded
-
     def get_args(self):
         """
         Returns the list of encoded job arguments. The order of this list corresponds to the
@@ -866,7 +868,7 @@ class JobArguments(object):
         return [
             self.task_cls.__module__,
             self.task_cls.__name__,
-            self.encode_params(self.task_params),
+            self.encode_string(self.task_params),
             self.encode_list(self.branches),
             self.encode_bool(self.auto_retry),
             self.encode_list(self.dashboard_data),
