@@ -21,11 +21,11 @@ import six
 
 from law.config import Config
 from law.task.base import Task
-from law.task.proxy import ProxyTask, get_proxy_attribute
+from law.task.proxy import ProxyTask, ProxyCommand, get_proxy_attribute
 from law.target.local import LocalDirectoryTarget
 from law.target.collection import TargetCollection
 from law.parameter import NO_STR
-from law.parser import global_cmdline_args, root_task
+from law.parser import root_task
 from law.util import (
     colored, is_pattern, multi_match, mask_struct, map_struct, interruptable_popen, patch_object,
     flatten,
@@ -72,6 +72,14 @@ class StageInfo(object):
         self.stage_dir = stage_dir
         self.stage_targets = stage_targets
 
+    def __str__(self):
+        tmpl = "{}.{} object at {}:\n  targets      : {}\n  stage_dir    : {}\n  stage_targets: {}"
+        return tmpl.format(self.__class__.__module__, self.__class__.__name__, hex(id(self)),
+            self.targets, self.stage_dir.path, self.stage_targets)
+
+    def __repr__(self):
+        return str(self)
+
 
 @six.add_metaclass(ABCMeta)
 class Sandbox(object):
@@ -231,13 +239,7 @@ class Sandbox(object):
     def _expand_volume(self, vol, bin_dir=None, python_dir=None):
         def replace(vol, name, repl):
             # warn about the deprecation of the legacy format "${name}" (until v0.1)
-            var = "{" + name + "}"
-            old_var = "${" + name + "}"
-            if old_var in vol:
-                logger.warning("the volume definition '{}' contains a deprecated variable '{}' "
-                    "which will be removed in the future, please use '{}' instead".format(
-                        vol, old_var, var))
-                vol = vol.replace(old_var, repl)
+            var = "{{LAW_FORWARD_" + name + "}}"
             vol = vol.replace(var, repl)
             return vol
 
@@ -270,17 +272,9 @@ class SandboxProxy(ProxyTask):
     def sandbox_inst(self):
         return self.task.sandbox_inst
 
-    def proxy_cmd(self):
-        # start with "law run <module.task>"
-        cmd = ["law", "run", "{}.{}".format(self.task.__module__, self.task.__class__.__name__)]
-
-        # add cli args, exclude some parameters
-        cmd.extend(self.task.cli_args(exclude=self.task.exclude_params_sandbox))
-
-        # add global args, explicitely remove the --workers argument
-        cmd.extend(global_cmdline_args(exclude=[("--workers", 1)]))
-
-        return cmd
+    def create_proxy_cmd(self):
+        return ProxyCommand(self.task, exclude_task_args=self.task.exclude_params_sandbox,
+            exclude_global_args=["workers"])
 
     def run(self):
         # before_run hook
@@ -306,13 +300,13 @@ class SandboxProxy(ProxyTask):
             logger.debug("configured sandbox stage-out data")
 
         # create the actual command to run
-        cmd = self.sandbox_inst.cmd(self.proxy_cmd())
+        cmd = self.sandbox_inst.cmd(self.create_proxy_cmd())
 
         # run with log section before and after actual run call
-        with self._run_log(cmd):
+        with self._run_context(cmd):
             code, out, err = self.sandbox_inst.run(cmd)
             if code != 0:
-                raise Exception("Sandbox '{}' failed with exit code {}".format(
+                raise Exception("sandbox '{}' failed with exit code {}".format(
                     self.sandbox_inst.key, code))
 
         # actual stage_out
@@ -427,7 +421,7 @@ class SandboxProxy(ProxyTask):
         logger.info("staged-out {} file(s)".format(len(stageout_info.stage_dir.listdir())))
 
     @contextmanager
-    def _run_log(self, cmd=None):
+    def _run_context(self, cmd=None):
         def print_banner(msg, color):
             print("")
             print(colored(" {} ".format(msg).center(80, "="), color=color))
@@ -452,9 +446,8 @@ class SandboxProxy(ProxyTask):
 
 class SandboxTask(Task):
 
-    sandbox = luigi.Parameter(default=_current_sandbox[0] or NO_STR,
-        description="name of the sandbox to run the task in, default: $LAW_SANDBOX when set, "
-        "otherwise no default")
+    sandbox = luigi.Parameter(default=_current_sandbox[0] or NO_STR, description="name of the "
+        "sandbox to run the task in; default: $LAW_SANDBOX when set, otherwise empty")
 
     allow_empty_sandbox = False
     valid_sandboxes = ["*"]
@@ -524,7 +517,7 @@ class SandboxTask(Task):
         return mask_struct(self.sandbox_stagein(), staged_inputs, inputs)
 
     def _staged_output(self):
-        if not _sandbox_stagein_dir:
+        if not _sandbox_stageout_dir:
             raise Exception("LAW_SANDBOX_STAGEOUT_DIR must not be empty in a sandbox when target "
                 "stage-out is required")
 
