@@ -65,32 +65,15 @@ except ImportError:
 
 _local_fs = LocalFileSystem.default_instance
 
-global_retries = None
-
-global_retry_delay = None
-
 
 def retry(func=None, uri_cmd=None):
     def wrapper(func):
         @functools.wraps(func)
         def wrapper(self, *args, **kwargs):
-            if global_retries is not None:
-                retries = global_retries
-            else:
-                retries = kwargs.pop("retries", None)
-                if retries is None:
-                    retries = self.retries
-
-            if global_retry_delay is not None:
-                delay = global_retry_delay
-            else:
-                delay = kwargs.pop("retry_delay", None)
-                if delay is None:
-                    delay = self.retry_delay
-
-            random_base = kwargs.pop("random_base", None)
-            if random_base is None:
-                random_base = self.random_base
+            # get the number of retries, the retry delay and the random base setting
+            retries = kwargs.pop("retries", self.retries)
+            delay = kwargs.pop("retry_delay", self.retry_delay)
+            random_base = kwargs.pop("random_base", self.random_base)
 
             attempt = 0
             try:
@@ -356,7 +339,7 @@ class RemoteCache(object):
                 pass
 
     @classmethod
-    def parse_config(cls, section, config=None):
+    def parse_config(cls, section, config=None, overwrite=False):
         # reads a law config section and returns parsed file system configs
         cfg = Config.instance()
 
@@ -366,7 +349,9 @@ class RemoteCache(object):
         # helper to add a config value if it exists, extracted with a config parser method
         def add(option, func):
             cache_option = "cache_" + option
-            if option not in config and not cfg.is_missing_or_none(section, cache_option):
+            if cfg.is_missing_or_none(section, cache_option):
+                return
+            elif option not in config or overwrite:
                 config[option] = func(section, cache_option)
 
         def get_size(section, cache_option):
@@ -658,14 +643,14 @@ class RemoteFileSystem(FileSystem):
     local_fs = _local_fs
 
     @classmethod
-    def parse_config(cls, section, config=None):
+    def parse_config(cls, section, config=None, overwrite=False):
         config = super(RemoteFileSystem, cls).parse_config(section, config=config)
 
         cfg = Config.instance()
 
         # helper to add a config value if it exists, extracted with a config parser method
         def add(option, func):
-            if option not in config:
+            if option not in config or overwrite:
                 config[option] = func(section, option)
 
         def get_expanded_list(section, option):
@@ -688,30 +673,37 @@ class RemoteFileSystem(FileSystem):
             if not cfg.is_missing_or_none(section, option)
         })
 
-        # atomic contexts
+        # use atomic contexts per operation
         add("atomic_contexts", cfg.get_expanded_boolean)
 
-        # number of retries
+        # default number of retries
         add("retries", cfg.get_expanded_int)
 
-        # delay between retries
+        # default delay between retries
         add("retry_delay", get_time)
 
-        # random base selection
+        # default setting for the random base selection
         add("random_base", cfg.get_expanded_boolean)
 
-        # validation after copy
+        # default setting for validation after copy
         add("validate_copy", cfg.get_expanded_boolean)
 
+        # default setting for recursive directory removal
+        add("recursive_remove", cfg.get_expanded_boolean)
+
+        # default setting for using the cache
+        add("use_cache", cfg.get_expanded_boolean)
+
         # cache options
         if cfg.options(section, prefix="cache_"):
-            RemoteCache.parse_config(section, config.setdefault("cache_config", {}))
+            RemoteCache.parse_config(section, config.setdefault("cache_config", {}),
+                overwrite=overwrite)
 
         return config
 
     def __init__(self, base, bases=None, gfal_options=None, transfer_config=None,
             atomic_contexts=False, retries=0, retry_delay=0, random_base=True, validate_copy=False,
-            cache_config=None, local_fs=None, **kwargs):
+            recursive_remove=False, use_cache=False, cache_config=None, local_fs=None, **kwargs):
         FileSystem.__init__(self, **kwargs)
 
         # configure the gfal interface
@@ -721,6 +713,8 @@ class RemoteFileSystem(FileSystem):
 
         # store other configs
         self.validate_copy = validate_copy
+        self.recursive_remove = recursive_remove
+        self.use_cache = use_cache
 
         # set the cache when a cache root is set
         if cache_config and cache_config.get("root"):
@@ -742,6 +736,14 @@ class RemoteFileSystem(FileSystem):
     def __eq__(self, other):
         return self is other
 
+    def _eval_use_cache(self, use_cache):
+        if self.cache is None:
+            return False
+        elif use_cache is None:
+            return self.use_cache
+        else:
+            return bool(use_cache)
+
     def is_local(self, path):
         return get_scheme(path) == "file"
 
@@ -786,7 +788,10 @@ class RemoteFileSystem(FileSystem):
                 if not silent:
                     raise
 
-    def remove(self, path, recursive=True, silent=True, **kwargs):
+    def remove(self, path, recursive=None, silent=True, **kwargs):
+        if recursive is None:
+            recursive = self.recursive_remove
+
         if self.abspath(path) == "/":
             logger.warning("refused request to remove base directory of {!r}".format(self))
             return
@@ -919,18 +924,10 @@ class RemoteFileSystem(FileSystem):
 
         return dst
 
-    def _use_cache(self, cache):
-        if self.cache is None:
-            return False
-        elif cache is None:
-            return self.cache is not None
-        else:
-            return bool(cache)
-
     # generic copy with caching ability (local paths must have a "file://" scheme)
     def _cached_copy(self, src, dst, perm=None, cache=None, prefer_cache=False, validate=None,
             **kwargs):
-        cache = self._use_cache(cache)
+        cache = self._eval_use_cache(cache)
 
         # ensure absolute paths
         src = self.abspath(src)
@@ -1015,42 +1012,52 @@ class RemoteFileSystem(FileSystem):
             # simply copy and return the dst path
             return self._atomic_copy(src, dst, perm=perm, validate=validate, **kwargs)
 
-    def _prepare_dst_dir(self, src, dst, perm=None, **kwargs):
+    def _prepare_dst_dir(self, dst, src=None, perm=None, **kwargs):
+        """
+        Prepares the directory of a target located at *dst* and returns its full location as
+        specified below. *src* can be the location of a source file target, which is (e.g.) used by
+        a file copy or move operation. When *dst* is already a directory, calling this method has no
+        effect and the *dst* path is returned, optionally joined with the basename of *src*. When
+        *dst* is a file, *dst* path is returned unchanged. Otherwise, when *dst* does not exist yet,
+        it is interpreted as a file path and missing directories are created when
+        :py:attr:`create_file_dir` is *True*, using *perm* to set the directory permission. *dst* is
+        returned.
+        """
         rstat = self.exists(dst, stat=True)
-        src_base = os.path.basename(src)
 
         if rstat:
-            if self._s_isdir(rstat.st_mode):
-                full_dst = os.path.join(dst, src_base)
-            else:  # file
+            if self._s_isdir(rstat.st_mode) and src:
+                full_dst = os.path.join(dst, os.path.basename(src))
+            else:
                 full_dst = dst
+
         else:
-            # not existing, treat dst as a file name and create missing dirs
+            # interpret dst as a file name, create missing dirs
             dst_dir = self.dirname(dst)
-            if dst_dir and not self.exists(dst_dir):
+            if dst_dir and self.create_file_dir and not self.isdir(dst_dir):
                 self.mkdir(dst_dir, perm=perm, recursive=True, **kwargs)
             full_dst = dst
 
         return full_dst
 
-    def copy(self, src, dst, perm=None, dir_perm=None, cache=None, validate=None, **kwargs):
+    def copy(self, src, dst, perm=None, dir_perm=None, **kwargs):
         # dst might be an existing directory
         if dst:
             if self.is_local(dst):
-                self.local_fs._prepare_dst_dir(src, dst, perm=dir_perm)
+                self.local_fs._prepare_dst_dir(dst, src=src, perm=dir_perm)
             else:
-                self._prepare_dst_dir(src, dst, perm=dir_perm, **kwargs)
+                self._prepare_dst_dir(dst, src=src, perm=dir_perm, **kwargs)
 
         # copy the file
-        return self._cached_copy(src, dst, perm=perm, cache=cache, validate=validate, **kwargs)
+        return self._cached_copy(src, dst, perm=perm, **kwargs)
 
-    def move(self, src, dst, perm=None, dir_perm=None, validate=None, **kwargs):
+    def move(self, src, dst, perm=None, dir_perm=None, **kwargs):
         if not dst:
             raise Exception("move requires dst to be set")
 
         # copy the file
-        dst = self.copy(src, dst, perm=perm, dir_perm=dir_perm, cache=False, validate=validate,
-            **kwargs)
+        kwargs.pop("cache", None)
+        dst = self.copy(src, dst, perm=perm, dir_perm=dir_perm, cache=False, **kwargs)
 
         # remove the src
         if self.is_local(src):
@@ -1061,7 +1068,7 @@ class RemoteFileSystem(FileSystem):
         return dst
 
     def open(self, path, mode, cache=None, **kwargs):
-        cache = self._use_cache(cache)
+        cache = self._eval_use_cache(cache)
 
         yield_path = kwargs.pop("_yield_path", False)
 
@@ -1071,14 +1078,13 @@ class RemoteFileSystem(FileSystem):
         if mode == "r":
             if cache:
                 lpath = self._cached_copy(path, None, cache=True, **kwargs)
-                lpath = remove_scheme(lpath)
             else:
                 tmp = LocalFileTarget(is_tmp=self.ext(path, n=0) or True)
-                lpath = tmp.path
-                self.copy(path, add_scheme(lpath, "file"), cache=False, **kwargs)
+                lpath = self.copy(path, tmp.uri(), cache=False, **kwargs)
+            lpath = remove_scheme(lpath)
 
             def cleanup():
-                if not cache and tmp.exists():
+                if not cache and tmp and tmp.exists():
                     tmp.remove()
 
             f = lpath if yield_path else open(lpath, "r")
@@ -1095,7 +1101,7 @@ class RemoteFileSystem(FileSystem):
             def copy_and_cleanup():
                 try:
                     if tmp.exists():
-                        self._cached_copy(add_scheme(lpath, "file"), path, cache=cache, **kwargs)
+                        self.copy(tmp.uri(), path, cache=cache, **kwargs)
                 finally:
                     cleanup()
 
@@ -1244,10 +1250,15 @@ class RemoteFileProxy(object):
         return self.f.__enter__() if self.is_file else self.f
 
     def __exit__(self, exc_type, exc_value, traceback):
+        # when an exception was raised, the context was not successful
+        success = exc_type is None
+
+        # invoke the exit of the file object
+        # when its return value is True, it overwrites the success flag
         if getattr(self.f, "__exit__", None) is not None:
-            success = self.f.__exit__(exc_type, exc_value, traceback)
-        else:
-            success = exc_type is None
+            exit_ret = self.f.__exit__(exc_type, exc_value, traceback)
+            if exit_ret is True:
+                success = True
 
         if success:
             if callable(self.success_fn):
