@@ -11,11 +11,10 @@ __all__ = ["Task", "WrapperTask", "ExternalTask"]
 import sys
 import socket
 import time
-import math
 import logging
 from collections import OrderedDict
 from contextlib import contextmanager
-from abc import abstractmethod
+from abc import ABCMeta, abstractmethod
 from inspect import getargspec
 
 import luigi
@@ -25,9 +24,10 @@ from law.config import Config
 from law.parameter import NO_STR, CSVParameter
 from law.target.file import localize_file_targets
 from law.parser import root_task, global_cmdline_values
+from law.logger import setup_logger
 from law.util import (
-    abort, common_task_params, colored, uncolored, make_list, multi_match, flatten, BaseStream,
-    human_duration, patch_object,
+    no_value, abort, common_task_params, colored, uncolored, make_list, multi_match, flatten,
+    BaseStream, human_duration, patch_object, round_discrete, classproperty,
 )
 
 
@@ -37,18 +37,41 @@ logger = logging.getLogger(__name__)
 class BaseRegister(luigi.task_register.Register):
 
     def __new__(metacls, classname, bases, classdict):
-        # default attributes
+        # default attributes, irrespective of inheritance
         classdict.setdefault("exclude_index", False)
 
-        # union "exclude_params_*" sets with those of all base classes
+        # unite "exclude_params_*" sets with those of all base classes
         for base in bases:
             for attr, base_params in vars(base).items():
-                if isinstance(base_params, set) and attr.startswith("exclude_params_"):
+                if attr.startswith("exclude_params_") and isinstance(base_params, set):
                     params = classdict.setdefault(attr, set())
                     if isinstance(params, set):
                         params.update(base_params)
 
-        return super(BaseRegister, metacls).__new__(metacls, classname, bases, classdict)
+        # remove those parameter names from "exclude_params_*" sets which are explicitly
+        # listed in corresponding "include_params_*" sets defined on the class itself
+        for attr, include_params in classdict.items():
+            if attr.startswith("include_params_") and isinstance(include_params, set):
+                exclude_attr = "exclude" + attr[len("include"):]
+                if exclude_attr in classdict and isinstance(classdict[exclude_attr], set):
+                    classdict[exclude_attr] -= include_params
+
+        # create the class
+        cls = ABCMeta.__new__(metacls, classname, bases, classdict)
+
+        # default attributes, apart from inheritance
+        if getattr(cls, "update_register", None) is None:
+            cls.update_register = False
+
+        # deregister when requested
+        if cls.update_register:
+            cls.deregister()
+
+        # add to register (mimic luigi.task_register.Register.__new__)
+        cls._namespace_at_class_time = metacls._get_namespace(cls.__module__)
+        metacls._reg.append(cls)
+
+        return cls
 
 
 @six.add_metaclass(BaseRegister)
@@ -66,11 +89,44 @@ class BaseTask(luigi.Task):
             host = socket.gethostname().partition(".")[0]
         return "{}_{}".format(host, name)
 
+    @classmethod
+    def deregister(cls, task_cls=None):
+        """
+        Removes a task class *task_cls* from the luigi task register. When *None*, *this* class is
+        used. Task family strings and patterns are accepted as well. *True* is returned when at
+        least one class was successfully removed, and *False* otherwise.
+        """
+        # always compare task families
+        if task_cls is None:
+            task_family = cls.task_family
+        elif isinstance(task_cls, six.string_types):
+            task_family = task_cls
+        else:
+            task_family = task_cls.task_family
+
+        success = False
+
+        # remove from the register
+        i = -1
+        while True:
+            i += 1
+            if i >= len(Register._reg):
+                break
+            registered_cls = Register._reg[i]
+
+            if multi_match(registered_cls.task_family, task_family, mode=any):
+                Register._reg.pop(i)
+                i -= 1
+                success = True
+                logger.debug("removed task class {} from register".format(registered_cls))
+
+        return success
+
     @classmethod
     def get_param_values(cls, *args, **kwargs):
         values = super(BaseTask, cls).get_param_values(*args, **kwargs)
         if callable(cls.modify_param_values):
-            return cls.modify_param_values(OrderedDict(values)).items()
+            return list(cls.modify_param_values(OrderedDict(values)).items())
         else:
             return values
 
@@ -120,6 +176,16 @@ class BaseTask(luigi.Task):
 
         return params
 
+    @classmethod
+    def get_logger_name(cls):
+        return "{}.{}".format(cls.__module__, cls.__name__)
+
+    @classproperty
+    def logger(cls):
+        name = cls.get_logger_name()
+        is_configured = name in logging.root.manager.loggerDict
+        return logging.getLogger(name) if is_configured else setup_logger(name)
+
     def complete(self):
         outputs = [t for t in flatten(self.output()) if not t.optional]
 
@@ -177,15 +243,13 @@ class BaseTask(luigi.Task):
         if replace is None:
             replace = {}
 
-        args = []
+        args = OrderedDict()
         for name, param in self.get_params():
             if multi_match(name, exclude, any):
                 continue
             raw = replace.get(name, getattr(self, name))
             val = param.serialize(raw)
-            arg = "--{}".format(name.replace("_", "-"))
-            # TODO: why does quote_cmd([val]) fail while str(val) doesn't
-            args.extend([arg, str(val)])
+            args["--" + name.replace("_", "-")] = str(val)
 
         return args
 
@@ -205,7 +269,7 @@ class Register(BaseRegister):
             if value:
                 skip_abort = False
                 try:
-                    logger.debug("evaluating interactive parameter '{}' with value '{}'".format(
+                    logger.debug("evaluating interactive parameter '{}' with value {}".format(
                         param, value))
                     skip_abort = getattr(inst, "_" + param)(value)
                 except KeyboardInterrupt:
@@ -221,7 +285,7 @@ class Register(BaseRegister):
 @six.add_metaclass(Register)
 class Task(BaseTask):
 
-    log_file = luigi.Parameter(default=NO_STR, significant=False, description="a custom log file, "
+    log_file = luigi.Parameter(default=NO_STR, significant=False, description="a custom log file; "
         "default: <task.default_log_file>")
     print_deps = CSVParameter(default=(), significant=False, description="print task dependencies "
         "but do not run any task; this CSV parameter accepts a single integer value which sets the "
@@ -233,7 +297,7 @@ class Task(BaseTask):
         "'')")
     print_output = CSVParameter(default=(), significant=False, description="print a flat list of "
         "output targets but do not run any task; this CSV parameter accepts a single integer value "
-        "which sets the task recursion depth (0 means non-recursive")
+        "which sets the task recursion depth (0 means non-recursive)")
     remove_output = CSVParameter(default=(), significant=False, description="remove task outputs "
         "but do not run any task; this CSV parameter accepts up to three values: 1. the task "
         "recursion depth (0 means non-recursive), 2. one of the modes 'i' (interactive), 'a' "
@@ -282,15 +346,17 @@ class Task(BaseTask):
     def is_root_task(self):
         return root_task() == self
 
-    def publish_message(self, *args):
-        msg = " ".join(str(arg) for arg in args)
-        print(msg)
+    def publish_message(self, msg, scheduler=True):
+        msg = str(msg)
+
+        sys.stdout.write(msg + "\n")
         sys.stdout.flush()
 
-        self._publish_message(*args)
+        if scheduler:
+            self._publish_message(msg)
 
-    def _publish_message(self, *args):
-        msg = " ".join(str(arg) for arg in args)
+    def _publish_message(self, msg):
+        msg = str(msg)
 
         # add to message cache and handle overflow
         msg = uncolored(msg)
@@ -309,8 +375,9 @@ class Task(BaseTask):
         return TaskMessageStream(self, *args, **kwargs)
 
     @contextmanager
-    def publish_step(self, msg, success_message="done", fail_message="failed", runtime=False):
-        self.publish_message(msg)
+    def publish_step(self, msg, success_message="done", fail_message="failed", runtime=True,
+            scheduler=True):
+        self.publish_message(msg, scheduler=scheduler)
         success = False
         t0 = time.time()
         try:
@@ -321,10 +388,10 @@ class Task(BaseTask):
             if runtime:
                 diff = time.time() - t0
                 msg = "{} (took {})".format(msg, human_duration(seconds=diff))
-            self.publish_message(msg)
+            self.publish_message(msg, scheduler=scheduler)
 
-    def publish_progress(self, percentage):
-        percentage = int(math.floor(percentage))
+    def publish_progress(self, percentage, precision=1):
+        percentage = int(round_discrete(percentage, precision, "floor"))
         if percentage != self._last_progress_percentage:
             self._last_progress_percentage = percentage
 
@@ -334,10 +401,10 @@ class Task(BaseTask):
                 logger.warning("set_progress_percentage not set, cannot send task progress to "
                     "scheduler")
 
-    def create_progress_callback(self, n_total, reach=(0, 100)):
+    def create_progress_callback(self, n_total, reach=(0, 100), precision=1):
         def make_callback(n, start, end):
             def callback(i):
-                self.publish_progress(start + (i + 1) / float(n) * (end - start))
+                self.publish_progress(start + (i + 1) / float(n) * (end - start), precision)
             return callback
 
         if isinstance(n_total, (list, tuple)):
@@ -358,18 +425,18 @@ class Task(BaseTask):
     def __repr__(self):
         return self.repr(color=False)
 
-    def repr(self, all_params=False, color=None):
+    def repr(self, all_params=False, color=None, **kwargs):
         if color is None:
             cfg = Config.instance()
             color = cfg.get_expanded_boolean("task", "colored_repr")
 
-        family = self._repr_family(self.get_task_family(), color=color)
+        family = self._repr_family(self.get_task_family(), color=color, **kwargs)
 
         parts = [
-            self._repr_param(name, value, color=color)
+            self._repr_param(name, value, color=color, **kwargs)
             for name, value in six.iteritems(self._repr_params(all_params=all_params))
         ] + [
-            self._repr_flag(flag, color=color)
+            self._repr_flag(flag, color=color, **kwargs)
             for flag in self._repr_flags()
         ]
 
@@ -383,22 +450,20 @@ class Task(BaseTask):
         return self.repr(all_params=all_params, color=True)
 
     def _repr_params(self, all_params=False):
-        # build key value pairs of all significant parameters
-        params = self.get_params()
-
+        # determine parameters to exclude
         exclude = set()
         if not all_params:
             exclude |= self.exclude_params_repr
             exclude |= self.inst_exclude_params_repr()
             exclude |= set(self.interactive_params)
 
-        values = OrderedDict()
-        for name, param in params:
+        # build a map "name -> value" for all significant parameters
+        params = OrderedDict()
+        for name, param in self.get_params():
             if param.significant and not multi_match(name, exclude):
-                value = getattr(self, name)
-                values[name] = param.serialize(value)
+                params[name] = getattr(self, name)
 
-        return values
+        return params
 
     def _repr_flags(self):
         return []
@@ -407,15 +472,21 @@ class Task(BaseTask):
         return set()
 
     @classmethod
-    def _repr_family(cls, family, color=False):
+    def _repr_family(cls, family, color=False, **kwargs):
         return colored(family, "green") if color else family
 
     @classmethod
-    def _repr_param(cls, name, value, color=False):
+    def _repr_param(cls, name, value, color=False, serialize=True, **kwargs):
+        # try to serialize first unless explicitly disabled
+        if serialize:
+            param = getattr(cls, name, no_value)
+            if param != no_value:
+                value = param.serialize(value)
+
         return "{}={}".format(colored(name, color="blue", style="bright") if color else name, value)
 
     @classmethod
-    def _repr_flag(cls, name, color=False):
+    def _repr_flag(cls, name, color=False, **kwargs):
         return colored(name, color="magenta") if color else name
 
     def _print_deps(self, args):
@@ -431,9 +502,7 @@ class Task(BaseTask):
         return remove_task_output(self, *args)
 
     def _fetch_output(self, args):
-        import law.target.remote as ltr
-        with patch_object(ltr, "global_retries", 0, lock=True):
-            return fetch_task_output(self, *args)
+        return fetch_task_output(self, *args)
 
     def localize_input(self, *args, **kwargs):
         return localize_file_targets(self.input(), *args, **kwargs)
@@ -472,16 +541,17 @@ class ExternalTask(Task):
 
 class TaskMessageStream(BaseStream):
 
-    def __init__(self, task, stdout=True):
+    def __init__(self, task, stdout=True, scheduler=True):
         super(TaskMessageStream, self).__init__()
         self.task = task
         self.stdout = stdout
+        self.scheduler = scheduler
 
-    def _write(self, *args):
+    def _write(self, msg):
         if self.stdout:
-            self.task.publish_message(*args)
-        else:
-            self.task._publish_message(*args)
+            self.task.publish_message(msg, scheduler=self.scheduler)
+        elif self.scheduler:
+            self.task._publish_message(msg)
 
 
 # trailing imports
