@@ -6,15 +6,16 @@ Helpful utility functions.
 
 
 __all__ = [
-    "default_lock", "io_lock", "no_value", "rel_path", "law_src_path", "law_home_path", "law_run",
-    "print_err", "abort", "is_number", "try_int", "str_to_int", "common_task_params", "colored",
-    "uncolored", "query_choice", "is_pattern", "brace_expand", "multi_match", "is_lazy_iterable",
-    "make_list", "make_tuple", "flatten", "merge_dicts", "which", "map_verbose", "map_struct",
-    "mask_struct", "tmp_file", "interruptable_popen", "readable_popen", "create_hash",
-    "copy_no_perm", "makedirs_perm", "user_owns_file", "iter_chunks", "human_bytes", "parse_bytes",
-    "human_duration", "human_time_diff", "parse_duration", "is_file_exists_error",
-    "check_bool_flag", "send_mail", "ShorthandDict", "open_compat", "patch_object",
-    "join_generators", "quote_cmd", "BaseStream", "TeeStream", "FilteredStream",
+    "default_lock", "io_lock", "console_lock", "no_value", "rel_path", "law_src_path",
+    "law_home_path", "law_run", "print_err", "abort", "is_number", "try_int", "round_discrete",
+    "str_to_int", "flag_to_bool", "common_task_params", "colored", "uncolored", "query_choice",
+    "is_pattern", "brace_expand", "range_expand", "range_join", "multi_match", "is_iterable",
+    "is_lazy_iterable", "make_list", "make_tuple", "make_unique", "flatten", "merge_dicts", "which",
+    "map_verbose", "map_struct", "mask_struct", "tmp_file", "interruptable_popen", "readable_popen",
+    "create_hash", "copy_no_perm", "makedirs_perm", "user_owns_file", "iter_chunks", "human_bytes",
+    "parse_bytes", "human_duration", "human_time_diff", "parse_duration", "is_file_exists_error",
+    "send_mail", "DotDict", "ShorthandDict", "open_compat", "patch_object", "join_generators",
+    "quote_cmd", "classproperty", "BaseStream", "TeeStream", "FilteredStream",
 ]
 
 
@@ -34,6 +35,7 @@ import copy
 import collections
 import contextlib
 import smtplib
+import time
 import datetime
 import random
 import threading
@@ -45,6 +47,7 @@ import six
 
 try:
     import ipykernel
+    import ipykernel.iostream
 except ImportError:
     ipykernel = None
 
@@ -54,18 +57,29 @@ logger = logging.getLogger(__name__)
 # some globally usable thread locks
 default_lock = threading.Lock()
 io_lock = threading.Lock()
+console_lock = threading.Lock()
 
 
 class NoValue(object):
 
+    _instance = None
+
+    def __new__(cls, *args, **kwargs):
+        if cls._instance is None:
+            cls._instance = super(NoValue, cls).__new__(cls, *args, **kwargs)
+        return cls._instance
+
     def __bool__(self):
         return False
 
     def __nonzero__(self):
         return False
 
+    def __str__(self):
+        return "{}.no_value".format(self.__module__)
 
-#: Unique dummy value that evaluates to *False*.
+
+#: Unique dummy value that is used to denote missing values and always evaluates to *False*.
 no_value = NoValue()
 
 
@@ -161,6 +175,40 @@ def try_int(n):
     return n_int if n == n_int else n
 
 
+def round_discrete(n, base=1., round_fn=round):
+    """ round_discrete(n, base=1.0, round_fn="round")
+    Rounds a number *n* to a discrete *base*. *round_fn* can be a function used for rounding and
+    defaults to the built-in ``round`` function. It also accepts string values ``"round"``,
+    ``"floor"`` and ``"ceil"`` which are resolved to the corresponding math functions. Example:
+
+    .. code-block:: python
+
+        round_discrete(17, 5)
+        # -> 15.
+
+        round_discrete(17, 2.5)
+        # -> 17.5
+
+        round_discrete(17, 2.5)
+        # -> 17.5
+
+        round_discrete(17, 2.5, math.floor)
+        round_discrete(17, 2.5, "floor")
+        # -> 15.0
+    """
+    if isinstance(round_fn, six.string_types):
+        if round_fn == "round":
+            round_fn = round
+        elif round_fn == "floor":
+            round_fn = math.floor
+        elif round_fn == "ceil":
+            round_fn = math.ceil
+        else:
+            raise ValueError("unknown round function '{}'".format(round_fn))
+
+    return base * round_fn(float(n) / base)
+
+
 def str_to_int(s):
     """
     Converts a string *s* into an integer under consideration of binary, octal, decimal and
@@ -172,6 +220,27 @@ def str_to_int(s):
     return int(s, base=base)
 
 
+def flag_to_bool(s, silent=False):
+    """
+    Takes a string flag *s* and returns whether it evaluates to *True* (values ``"1"``, ``"true"``
+    ``"yes"``, ``"on"``, case-insensitive) or *False* (values ``"0"``, ``"false"``, `"no"``,
+    ``"off"``, case-insensitive). When *s* is already a boolean, it is returned unchanged. An error
+    is thrown when *s* is neither of the allowed values and *silent* is *False*. Otherwise, *None*
+    is returned.
+    """
+    if isinstance(s, bool):
+        return s
+    elif isinstance(s, six.string_types):
+        if s.lower() in ("true", "1", "yes", "on"):
+            return True
+        elif s.lower() in ("false", "0", "no", "off"):
+            return False
+    elif silent:
+        return None
+    else:
+        raise ValueError("cannot convert to bool: {}".format(s))
+
+
 def common_task_params(task_instance, task_cls):
     """
     Returns the parameters that are common between a *task_instance* and a *task_cls* in a
@@ -256,7 +325,7 @@ def colored(msg, color=None, background=None, style=None, force=False):
         except:
             pass
 
-        if not tty and getattr(ipykernel, "iostream", None):
+        if not tty and ipykernel is not None:
             ipy = isinstance(sys.stdout, ipykernel.iostream.OutStream)
 
         if not tty and not ipy:
@@ -315,7 +384,7 @@ def query_choice(msg, choices, default=None, descriptions=None, lower=True):
     choice = None
     while choice not in _choices:
         if choice is not None:
-            print("invalid choice: '{}'\n".format(choice))
+            print("invalid choice: '{}'".format(choice))
         choice = six.moves.input(msg)
         if default is not None and choice == "":
             choice = default
@@ -360,8 +429,8 @@ def brace_expand(s, split_csv=False):
         # -> ["A1B", "A2B", "C3D"]
     """
     # first, replace escaped braces
-    br_open = "__LAW_BRACE_OPEN__"
-    br_close = "__LAW_BRACE_CLOSE__"
+    br_open = "__law_brace_open__"
+    br_close = "__law_brace_close__"
     s = s.replace(r"\{", br_open).replace(r"\}", br_close)
 
     # compile the expression that finds brace statements
@@ -370,7 +439,7 @@ def brace_expand(s, split_csv=False):
     # take into account csv splitting
     if split_csv:
         # replace commas in brace statements to avoid splitting
-        br_comma = "__LAW_BRACE_COMMA__"
+        br_comma = "__law_brace_comma__"
         _s = cre.sub(lambda m: m.group(0).replace(",", br_comma), s)
         # split by real csv commas and start recursion when a comma was found, otherwise continue
         parts = _s.split(",")
@@ -405,10 +474,124 @@ def brace_expand(s, split_csv=False):
     return res
 
 
+def range_expand(s, min_value=None, max_value=None):
+    """
+    Takes a string or sequence of strings denoting single positive numbers or ranges of positive
+    numbers such as ``"1-3"`` and returns a list of all selected values, inclusive at both edges.
+    One sided range expressions such as ``"-4"`` or ``"4-"`` are also expanded but they require
+    *min_value* and *max_value* to be set (an exception is raised otherwise). Also, when a
+    *min_value* (*max_value*) is given, no value in the returned list of numbers can be smaller
+    (larger). Example:
+
+    .. code-block:: python
+
+        range_expand("5-8")
+        # -> [5, 6, 7, 8]
+
+        range_expand(["5-8", "10"])
+        # -> [5, 6, 7, 8, 10]
+
+        range_expand(["5-8", "10-"])
+        # -> Exception, no max_value set
+
+        range_expand(["5-8", "10-"], max_value=12)
+        # -> [5, 6, 7, 8, 10, 11, 12]
+    """
+    def to_int(v, s=None):
+        try:
+            return int(v)
+        except ValueError:
+            raise ValueError("invalid number or range '{}'".format(v if s is None else s))
+
+    numbers = []
+    for s in make_list(s):
+        s = str(s)
+        if "-" in s:
+            # split into start and stop value and handle empty values
+            start, stop = s.split("-", 1)
+            if not start:
+                if min_value is None:
+                    raise Exception("range '{}' with missing start value requires min_value to be "
+                        "set".format(s))
+                start = min_value
+            if not stop:
+                if max_value is None:
+                    raise Exception("range '{}' with missing stop value requires max_value to be "
+                        "set".format(s))
+                stop = max_value
+
+            # convert to integers and potentially swap
+            start = to_int(start)
+            stop = to_int(stop)
+            if start > stop:
+                start, stop = stop, start
+
+            # add numbers
+            numbers.extend(range(start, stop + 1))
+        else:
+            # no "-" in s, assume it to be a number
+            numbers.append(to_int(s))
+
+    # apply min and max cuts when given
+    if min_value is not None:
+        numbers = [n for n in numbers if n >= min_value]
+    if max_value is not None:
+        numbers = [n for n in numbers if n <= max_value]
+
+    # remove duplicates preserving the order
+    numbers = make_unique(numbers)
+
+    return numbers
+
+
+def range_join(numbers, to_str=False):
+    """
+    Takes a sequence of positive integer numbers and returns a sequence 1- and 2-tuples, denoting
+    either single numbers or inclusive start and stop values of possible ranges. When *to_str* is
+    *True*, a string is returned in a format consistent to :py:func:`range_expand`. Example:
+
+    .. code-block:: python
+
+        range_join([1, 2, 3, 5])
+        # -> [(1, 3), (5,)]
+
+        range_join([1, 2, 3, 5, 7, 8, 9])
+        # -> [(1, 3), (5,), (7, 9)]
+
+        range_join([1, 2, 3, 5, 7, 8, 9], to_str=True)
+        # -> "1-3,5,7-9"
+    """
+    if not numbers:
+        return "" if to_str else []
+
+    # type check and sort
+    for n in numbers:
+        if not isinstance(n, six.integer_types):
+            raise TypeError("cannot handle non-integer value '{}' in numbers to join".format(n))
+    numbers = sorted(numbers)
+
+    # iterate through numbers, keep track of last starts and stops and fill a list of range tuples
+    ranges = []
+    start = stop = numbers[0]
+    for n in numbers[1:]:
+        if n == stop + 1:
+            stop += 1
+        else:
+            ranges.append((start,) if start == stop else (start, stop))
+            start = stop = n
+    ranges.append((start,) if start == stop else (start, stop))
+
+    # convert to string representation
+    if to_str:
+        ranges = ",".join(("{}" if len(r) == 1 else "{}-{}").format(*r) for r in ranges)
+
+    return ranges
+
+
 def multi_match(name, patterns, mode=any, regex=False):
     """
     Compares *name* to multiple *patterns* and returns *True* in case of at least one match (*mode*
-    = *any*, the default), or in case all patterns matched (*mode* = *all*). Otherwise, *False* is
+    = *any*, the default), or in case all patterns match (*mode* = *all*). Otherwise, *False* is
     returned. When *regex* is *True*, *re.match* is used instead of *fnmatch.fnmatch*.
     """
     patterns = make_list(patterns)
@@ -418,6 +601,17 @@ def multi_match(name, patterns, mode=any, regex=False):
         return mode(re.match(pattern, name) for pattern in patterns)
 
 
+def is_iterable(obj):
+    """
+    Returns *True* when an object *obj* is iterable and *False* otherwise.
+    """
+    try:
+        iter(obj)
+    except Exception:
+        return False
+    return True
+
+
 def is_lazy_iterable(obj):
     """
     Returns whether *obj* is iterable lazily, such as generators, range objects, maps, etc.
@@ -458,6 +652,24 @@ def make_tuple(obj, cast=True):
         return (obj,)
 
 
+def make_unique(obj):
+    """
+    Takes a list or tuple *obj*, removes duplicate elements in order of their appearance and returns
+    the sequence of remaining, unique elements. The sequence type is preserved. When *obj* is
+    neither a list nor a tuple, but iterable, a list is returned. Otherwise, a *TypeError* is
+    raised.
+    """
+    if not isinstance(obj, (list, tuple)):
+        if is_iterable(obj):
+            obj = list(obj)
+        else:
+            raise TypeError("object is neither list, tuple, nor generic iterable")
+
+    ret = sorted(obj.__class__(set(obj)), key=lambda elem: obj.index(elem))
+
+    return obj.__class__(ret) if isinstance(obj, tuple) else ret
+
+
 def flatten(*structs, **kwargs):
     """ flatten(*structs, flatten_dict=True, flatten_list=True, flatten_tuple=True, flatten_set=True)
     Takes one or multiple complex structured objects *structs*, flattens them, and returns a single
@@ -758,20 +970,56 @@ def tmp_file(*args, **kwargs):
 
 
 def interruptable_popen(*args, **kwargs):
-    """
-    Shorthand to :py:class:`Popen` followed by :py:meth:`Popen.communicate`. All *args* and *kwargs*
-    are forwatded to the :py:class:`Popen` constructor. The return code, standard output and
-    standard error are returned in a tuple. The call :py:meth:`Popen.communicate` is interruptable
-    by the user.
-    """
-    kwargs["preexec_fn"] = os.setsid
+    """ interruptable_popen(*args, interrupt_callback=None, kill_timeout=None, **kwargs)
+    Shorthand to :py:class:`Popen` followed by :py:meth:`Popen.communicate` which can be interrupted
+    by *KeyboardInterrupt*. The return code, standard output and standard error are returned in a
+    3-tuple.
 
+    *interrupt_callback* can be a function, accepting the process instance as an argument, that is
+    called immediately after a *KeyboardInterrupt* occurs. After that, a SIGTERM signal is send to
+    the subprocess to allow it to gracefully shutdown.
+
+    When *kill_timeout* is set, and the process is still alive after that period (in seconds), a
+    SIGKILL signal is sent to force the process termination.
+
+    All other *args* and *kwargs* are forwarded to the :py:class:`Popen` constructor.
+    """
+    # get kwargs not being passed to Popen
+    interrupt_callback = kwargs.pop("interrupt_callback", None)
+    kill_timeout = kwargs.pop("kill_timeout", None)
+
+    # start the subprocess in a new process group
+    kwargs["preexec_fn"] = os.setsid
     p = subprocess.Popen(*args, **kwargs)
 
+    # handle interrupts
     try:
         out, err = p.communicate()
     except KeyboardInterrupt:
-        os.killpg(os.getpgid(p.pid), signal.SIGTERM)
+        # allow the interrupt_callback to perform a custom process termination
+        if callable(interrupt_callback):
+            interrupt_callback(p)
+
+        # when the process is still alive, send SIGTERM to gracefully terminate it
+        pgid = os.getpgid(p.pid)
+        if p.poll() is None:
+            os.killpg(pgid, signal.SIGTERM)
+
+        # when a kill_timeout is set, and the process is still running after that period,
+        # send SIGKILL to force its termination
+        if kill_timeout is not None:
+            target_time = time.time() + kill_timeout
+            while target_time > time.time():
+                time.sleep(0.05)
+                if p.poll() is not None:
+                    # the process terminated, exit the loop
+                    break
+            else:
+                # check the status again to avoid race conditions
+                if p.poll() is None:
+                    os.killpg(pgid, signal.SIGKILL)
+
+        # transparently reraise
         raise
 
     if six.PY3:
@@ -785,19 +1033,24 @@ def interruptable_popen(*args, **kwargs):
 
 def readable_popen(*args, **kwargs):
     """
-    Shorthand to :py:class:`Popen` which yields the output live line-by-line. All *args* and
-    *kwargs* are forwatded to the :py:class:`Popen` constructor. When EOF is reached,
-    ``communicate()`` is called on the subprocess and it is yielded. Example:
+    Creates a :py:class:`Popen` object and a generator function yielding the output line-by-line as
+    it comes in. All *args* and *kwargs* are forwarded to the :py:class:`Popen` constructor.
+    Example:
 
     .. code-block:: python
 
-        for line in readable_popen(["some_executable", "--args"]):
-            if isinstance(line, str):
-                print(line)
-            else:
-                process = line
-                if process.returncode != 0:
-                    raise Exception("complain ...")
+        # create the popen object and line generator
+        p, lines = readable_popen(["some_executable", "--args"])
+
+        # loop through output lines as they come in
+        for line in lines:
+            print(line)
+
+        if p.returncode != 0:
+            raise Exception("complain ...")
+
+    ``communicate()`` is called automatically after the output iteration terminates which sets the
+    subprocess' *returncode* member.
     """
     # force pipes
     kwargs["stdout"] = subprocess.PIPE
@@ -805,14 +1058,16 @@ def readable_popen(*args, **kwargs):
 
     p = subprocess.Popen(*args, **kwargs)
 
-    for line in iter(lambda: p.stdout.readline(), ""):
-        if six.PY3:
-            line = line.decode("utf-8")
-        yield line.rstrip()
+    def line_gen():
+        for line in iter(lambda: p.stdout.readline(), ""):
+            if six.PY3:
+                line = line.decode("utf-8")
+            yield line.rstrip()
 
-    # yield the process itself in the end
-    p.communicate()
-    yield p
+        # communicate in the end
+        p.communicate()
+
+    return p, line_gen()
 
 
 def create_hash(inp, l=10, algo="sha256"):
@@ -1293,15 +1548,6 @@ def is_file_exists_error(e):
         return isinstance(e, OSError) and e.errno == 17
 
 
-def check_bool_flag(s):
-    """
-    Takes a string flag *s* and returns whether it evaluates to *True* (values ``"1"``, ``"true"``
-    and ``"yes"``, case-insensitive) or *False* (any other value). When *s* is not a string, *s* is
-    returned unchanged.
-    """
-    return s.lower() in ("1", "yes", "true") if isinstance(s, six.string_types) else s
-
-
 def send_mail(recipient, sender, subject="", content="", smtp_host="127.0.0.1", smtp_port=25):
     """
     Lightweight mail functionality. Sends an mail from *sender* to *recipient* with *subject* and
@@ -1320,6 +1566,41 @@ def send_mail(recipient, sender, subject="", content="", smtp_host="127.0.0.1",
     return True
 
 
+class DotDict(collections.OrderedDict):
+    """
+    Subclass of *OrderedDict* that provides read access for items via attributes by implementing
+    ``__getattr__``. In case a item is accessed via attribute and it does not exist, an
+    *AttriuteError* is raised rather than a *KeyError*. Example:
+
+    .. code-block:: python
+
+       d = DotDict()
+       d["foo"] = 1
+
+       print(d["foo"])
+       # => 1
+
+       print(d.foo)
+       # => 1
+
+       print(d["bar"])
+       # => KeyError
+
+       print(d.bar)
+       # => AttributeError
+    """
+
+    def __getattr__(self, attr):
+        if attr == "_OrderedDict__root":
+            return super(DotDict, self).__getattr__(attr)
+        else:
+            try:
+                return self[attr]
+            except KeyError:
+                raise AttributeError("'{}' object has no attribute '{}'".format(
+                    self.__class__.__name__, attr))
+
+
 class ShorthandDict(collections.OrderedDict):
     """
     Subclass of *OrderedDict* that implements ``__getattr__`` and ``__setattr__`` for a configurable
@@ -1328,7 +1609,6 @@ class ShorthandDict(collections.OrderedDict):
     .. code-block:: python
 
         MyDict(ShorthandDict):
-
             attributes = {"foo": 1, "bar": 2}
 
         d = MyDict(foo=9)
@@ -1486,6 +1766,41 @@ def quote_cmd(cmd):
     return " ".join(six.moves.shlex_quote(part) for part in cmd)
 
 
+class ClassPropertyDescriptor(object):
+    """
+    Generic descriptor class that is used by :py:func:`classproperty`. Setters are currently not
+    supported.
+    """
+
+    def __init__(self, fget, fset=None):
+        self.fget = fget
+        self.fset = fset
+
+    def __get__(self, obj, cls=None):
+        if cls is None:
+            cls = type(obj)
+
+        return self.fget.__get__(obj, cls)()
+
+    def __set__(self, obj, value):
+        if not self.fset:
+            raise AttributeError("can't set attribute")
+
+        type_ = type(obj)
+
+        return self.fset.__get__(obj, type_)(value)
+
+
+def classproperty(func):
+    """
+    Propety decorator for class-level methods.
+    """
+    if not isinstance(func, (classmethod, staticmethod)):
+        func = classmethod(func)
+
+    return ClassPropertyDescriptor(func)
+
+
 class BaseStream(object):
 
     FLUSH_AFTER_WRITE = True
@@ -1530,7 +1845,7 @@ class BaseStream(object):
 
 
 class TeeStream(BaseStream):
-    """
+    """ __init__(*consumers, mode="w")
     Multi-stream object that forwards calls to :py:meth:`write` and :py:meth:`flush` to all
     registered *consumer* streams. When a *consumer* is a string, it is interpreted as a file which
     is opened for writing (similar to *tee* in bash). Example:
@@ -1542,8 +1857,6 @@ class TeeStream(BaseStream):
     """
 
     def __init__(self, *consumers, **kwargs):
-        """ __init__(*consumers, mode="w")
-        """
         super(TeeStream, self).__init__()
 
         mode = kwargs.get("mode", "w")
