@@ -9,6 +9,7 @@ __all__ = ["BaseWorkflow", "workflow_property", "cached_workflow_property"]
 
 
 import sys
+import re
 import functools
 import logging
 from collections import OrderedDict
@@ -21,7 +22,9 @@ from law.task.base import Task, Register
 from law.task.proxy import ProxyTask, get_proxy_attribute
 from law.target.collection import TargetCollection
 from law.parameter import NO_STR, NO_INT, CSVParameter
-from law.util import no_value, make_list
+from law.util import (
+    no_value, make_list, iter_chunks, range_expand, range_join, create_hash, DotDict,
+)
 
 
 logger = logging.getLogger(__name__)
@@ -63,6 +66,8 @@ class BaseWorkflowProxy(ProxyTask):
                     self.run = run_func.__get__(self)
                     break
 
+        self._workflow_has_reset_branch_map = False
+
     def _get_task_attribute(self, name, fallback=False):
         """
         Return an attribute of the actual task named ``<workflow_type>_<name>``. When the attribute
@@ -95,8 +100,10 @@ class BaseWorkflowProxy(ProxyTask):
         Returns the default workflow requirements in an ordered dictionary, which is updated with
         the return value of the task's *workflow_requires* method.
         """
-        reqs = OrderedDict()
-        reqs.update(self.task.workflow_requires())
+        reqs = DotDict()
+        workflow_reqs = self.task.workflow_requires()
+        if workflow_reqs:
+            reqs.update(workflow_reqs)
         return reqs
 
     def output(self):
@@ -116,7 +123,7 @@ class BaseWorkflowProxy(ProxyTask):
         targets = luigi.task.getpaths(self.task.get_branch_tasks())
         collection = cls(targets, threshold=self.threshold(len(targets)))
 
-        return OrderedDict([("collection", collection)])
+        return DotDict([("collection", collection)])
 
     def threshold(self, n=None):
         """
@@ -133,6 +140,19 @@ class BaseWorkflowProxy(ProxyTask):
         acceptance = self.task.acceptance
         return (acceptance * n) if acceptance <= 1 else acceptance
 
+    def run(self):
+        """
+        Default run implementation that resets the branch map once if requested.
+        """
+        if self.task.reset_branch_map_before_run and not self._workflow_has_reset_branch_map:
+            self._workflow_has_reset_branch_map = True
+
+            # reset cached branch map, branch tasks and boundaries
+            self.task._branch_map = None
+            self.task._branch_tasks = None
+            self.task.start_branch = self.task._initial_start_branch
+            self.task.end_branch = self.task._initial_end_branch
+
 
 def workflow_property(func):
     """
@@ -163,13 +183,15 @@ def workflow_property(func):
     return property(wrapper)
 
 
-def cached_workflow_property(func=None, attr=None, setter=True):
+def cached_workflow_property(func=None, empty_value=no_value, attr=None, setter=True):
     """
     Decorator to declare an attribute that is stored only on a workflow and also cached for
     subsequent calls. Therefore, the decorated method is expected to (lazily) provide the value to
-    cache. The resulting value is stored as ``_workflow_cached_<func.__name__>`` on the workflow,
-    which can be overwritten by setting the *attr* argument. By default, a setter is provded to
-    overwrite the cache value. Set *setter* to *False* to disable this feature. Example:
+    cache. When the value is equal to *empty_value*, it is not cached and the next access to the
+    property will invoke the decorated method again. The resulting value is stored as
+    ``_workflow_cached_<func.__name__>`` on the workflow, which can be overwritten by setting the
+    *attr* argument. By default, a setter is provded to overwrite the cache value. Set *setter* to
+    *False* to disable this feature. Example:
 
     .. code-block:: python
 
@@ -190,7 +212,7 @@ def cached_workflow_property(func=None, attr=None, setter=True):
         @functools.wraps(func)
         def getter(self):
             wf = self.as_workflow()
-            if not hasattr(wf, _attr):
+            if getattr(wf, _attr, empty_value) == empty_value:
                 setattr(wf, _attr, func(wf))
             return getattr(wf, _attr)
 
@@ -216,6 +238,19 @@ class WorkflowRegister(Register):
         # this flag will define the classes in the mro to consider for instantiating the proxy
         cls._defined_workflow_proxy = "workflow_proxy_cls" in classdict
 
+    def __call__(cls, *args, **kwargs):
+        # complain when an abstract workflow method is not implemented
+        missing_methods = []
+        for attr in getattr(cls, "abstract_workflow_methods", set()):
+            full_attr = "{}_{}".format(cls.workflow_proxy_cls.workflow_type, attr)
+            if getattr(cls, full_attr, no_value) is no_value:
+                missing_methods.append(full_attr)
+        if missing_methods:
+            raise TypeError("cannot create instance of workflow task class '{}' with missing "
+                "method(s) {}".format(cls.__name__, ", ".join(missing_methods)))
+
+        return super(WorkflowRegister, cls).__call__(*args, **kwargs)
+
 
 @six.add_metaclass(WorkflowRegister)
 class BaseWorkflow(Task):
@@ -271,6 +306,13 @@ class BaseWorkflow(Task):
 
        Custom completion check that is used by the workflow's proxy when callable.
 
+    .. py:classattribute:: abstract_workflow_methods
+       type: set
+
+       Names of methods that have to be implemented by inheriting workflow classes, as checked by
+       the :py:class:`WorkflowRegister` meta class. The names of methods to implement must start
+       with the workflow type followed by an underscore.
+
     .. py:classattribute:: output_collection_cls
        type: TargetCollection
 
@@ -284,6 +326,12 @@ class BaseWorkflow(Task):
        Flag that denotes if this workflow is forced to use contiguous branch numbers, starting from
        0. If *False*, an exception is raised otherwise.
 
+    .. py:classattribute:: reset_branch_map_before_run
+       type: bool
+
+       Flag that denotes whether the branch map should be recreated from scratch before the run
+       method of the underlying workflow proxy is called.
+
     .. py:classattribute:: workflow_property
        type: function
 
@@ -325,35 +373,38 @@ class BaseWorkflow(Task):
     """
 
     workflow = luigi.Parameter(default=NO_STR, significant=False, description="the type of the "
-        "workflow to use, default: NO_STR")
+        "workflow to use; uses the first workflow type in the MRO when empty; default: empty")
     acceptance = luigi.FloatParameter(default=1.0, significant=False, description="number of "
-        "finished tasks to consider the task successful, relative fraction (<= 1) or absolute "
-        "value (> 1), default: 1.0")
+        "finished tasks to consider the task successful; relative fraction (<= 1) or absolute "
+        "value (> 1); default: 1.0")
     tolerance = luigi.FloatParameter(default=0.0, significant=False, description="number of failed "
-        "tasks to still consider the task successful, relative fraction (<= 1) or absolute value "
-        "(> 1), default: 0.0")
-    pilot = luigi.BoolParameter(significant=False, description="disable requirements of the "
-        "workflow to let branch tasks resolve requirements on their own")
+        "tasks to still consider the task successful; relative fraction (<= 1) or absolute value "
+        "(> 1); default: 0.0")
+    pilot = luigi.BoolParameter(default=False, significant=False, description="disable "
+        "requirements of the workflow to let branch tasks resolve requirements on their own; "
+        "default: False")
     branch = luigi.IntParameter(default=-1, description="the branch number/index to run this "
-        "task for, -1 means this task is the workflow, default: -1")
-    start_branch = luigi.IntParameter(default=NO_INT, description="the branch to start at, "
-        "default: 0")
-    end_branch = luigi.IntParameter(default=NO_INT, description="the branch to end at, NO_INT "
-        "means end, default: NO_INT")
-    branches = CSVParameter(default=(), description="list of branches to select, default: ()")
+        "task for; -1 means this task is the workflow; default: -1")
+    start_branch = luigi.IntParameter(default=NO_INT, description="the branch to start at; empty "
+        "value means first; default: empty")
+    end_branch = luigi.IntParameter(default=NO_INT, description="the branch to end at; empty value "
+        "means last; default: empty")
+    branches = CSVParameter(default=(), unique=True, description="list of branches to select; "
+        "has precedence over startBranch and endBranch when set; default: empty")
 
+    # configuration members
     workflow_proxy_cls = BaseWorkflowProxy
-
-    workflow_complete = None
-
     output_collection_cls = None
     force_contiguous_branches = False
+    reset_branch_map_before_run = False
+    workflow_run_decorators = None
+    workflow_complete = None
+    abstract_workflow_methods = set()
 
+    # accessible properties
     workflow_property = None
     cached_workflow_property = None
 
-    workflow_run_decorators = None
-
     exclude_index = True
 
     exclude_params_branch = {
@@ -389,6 +440,10 @@ class BaseWorkflow(Task):
         # cached attributes for branches
         self._workflow_task = None
 
+        # store original branch boundaries
+        self._initial_start_branch = self.start_branch
+        self._initial_end_branch = self.end_branch
+
     def __getattribute__(self, attr, proxy=True):
         return get_proxy_attribute(self, attr, proxy=proxy, super_cls=Task)
 
@@ -403,13 +458,21 @@ class BaseWorkflow(Task):
         return super(BaseWorkflow, self).cli_args(exclude=exclude, replace=replace)
 
     def _repr_params(self, *args, **kwargs):
-        values = super(BaseWorkflow, self)._repr_params(*args, **kwargs)
+        params = super(BaseWorkflow, self)._repr_params(*args, **kwargs)
 
         # when this is a workflow, add the workflow type
-        if self.is_workflow() and "workflow" not in values:
-            values["workflow"] = self.workflow
+        if self.is_workflow() and "workflow" not in params:
+            params["workflow"] = self.workflow
 
-        return values
+        return params
+
+    @classmethod
+    def _repr_param(cls, name, value, **kwargs):
+        if name == "branches":
+            value = range_join(value, to_str=True)
+            kwargs["serialize"] = False
+
+        return super(BaseWorkflow, cls)._repr_param(name, value, **kwargs)
 
     def is_branch(self):
         """
@@ -481,40 +544,20 @@ class BaseWorkflow(Task):
         if self.is_branch():
             raise Exception("calls to _reduce_branch_map are forbidden for branch tasks")
 
-        # reduce by start/end branch
-        for b in list(self._branch_map.keys()):
-            if not (self.start_branch <= b < self.end_branch):
-                del self._branch_map[b]
-
-        # reduce by branches
+        # when given, reduce by branches, otherwise by start/end branch
         if self.branches:
-            # helper to expand slices, e.g. "1-3" -> 1,2,3 or "4-" -> 4,5,6,...
-            def expand(b):
-                if "-" in str(b):
-                    parts = str(b).strip().split("-")
-                    if len(parts) == 2:
-                        start = int(parts[0]) if parts[0] else None
-                        end = int(parts[1]) if parts[1] else None
-                        return start, end
-                return int(b)
-
-            # determine branches to remove
-            remove_branches = sorted(list(self._branch_map.keys()))
-            for b in self.branches:
-                b = expand(b)
-                if isinstance(b, tuple):
-                    start = b[0] if b[0] is not None else min(remove_branches)
-                    end = b[1] if b[1] is not None else max(remove_branches)
-                    for b in range(start, end + 1):
-                        if b in remove_branches:
-                            remove_branches.remove(b)
-                else:
-                    if b in remove_branches:
-                        remove_branches.remove(b)
+            # create a set of branches to remove
+            remove_branches = set(self._branch_map.keys())
+            remove_branches -= set(range_expand(self.branches, min_value=min(remove_branches),
+                max_value=max(remove_branches)))
 
             # actual removal
             for b in remove_branches:
                 del self._branch_map[b]
+        else:
+            for b in list(self._branch_map.keys()):
+                if not (self.start_branch <= b < self.end_branch):
+                    del self._branch_map[b]
 
     def get_branch_map(self, reset_boundaries=True, reduce=True):
         """
@@ -574,18 +617,88 @@ class BaseWorkflow(Task):
         """
         if self.is_branch():
             return self.as_workflow().get_branch_tasks()
+
+        if self._branch_tasks is None:
+            branch_map = self.get_branch_map()
+            if branch_map is None:
+                raise AttributeError("workflow task '{}' requires a branch_map".format(self))
+
+            self._branch_tasks = OrderedDict()
+            for b in branch_map:
+                self._branch_tasks[b] = self.req(self, branch=b,
+                    _exclude=self.exclude_params_branch)
+
+        return self._branch_tasks
+
+    def get_branch_chunks(self, chunk_size):
+        """
+        Returns a list of chunks of branch numbers defined in this workflow with a certain
+        *chunk_size*. Example:
+
+        .. code-block:: python
+
+            wf = SomeWorkflowTask()  # has 8 branches
+            print(wf.get_branch_chunks(3))
+            # -> [[0, 1, 2], [3, 4, 5], [6, 7]]
+
+            wf2 = SomeWorkflowTask(end_branch=5)  # has 5 branches
+            print(wf2.get_branch_chunks(3))
+            # -> [[0, 1, 2], [3, 4]]
+        """
+        if self.is_branch():
+            return self.as_workflow().get_branch_chunks(chunk_size)
+
+        # get the branch map and create chunks of its branch values
+        branch_chunks = iter_chunks(self.branch_map.keys(), chunk_size)
+
+        return list(branch_chunks)
+
+    def get_all_branch_chunks(self, chunk_size, **kwargs):
+        """
+        Returns a list of chunks of all branch numbers of this workflow (i.e. without
+        *start_branch*, *end_branch* and *branches* parameters applied) with a certain *chunk_size*.
+        Internally, a new instance of this workflow is created using :py:meth:`BaseTask.req`,
+        forwarding all *kwargs*. Its *_exclude* list will contain ``["start_branch", "end_branch",
+        "branches"]`` in order to use all possible branch values. Example:
+
+        .. code-block:: python
+
+            wf = SomeWorkflowTask()  # has 8 branches
+            print(wf.get_all_branch_chunks(3))
+            # -> [[0, 1, 2], [3, 4, 5], [6, 7]]
+
+            wf2 = SomeWorkflowTask(end_branch=5)  # has 5 branches
+            print(wf2.get_all_branch_chunks(3))
+            # -> [[0, 1, 2], [3, 4, 5], [6, 7]]
+        """
+        if self.is_branch():
+            return self.as_workflow().get_all_branch_chunks(chunk_size, **kwargs)
+
+        # create a new instance
+        _exclude = make_list(kwargs.get("_exclude", []))
+        _exclude.extend(["start_branch", "end_branch", "branches"])
+        kwargs["_exclude"] = _exclude
+        inst = self.req(self, **kwargs)
+
+        # return its branch chunks
+        return inst.get_branch_chunks(chunk_size)
+
+    def get_branches_repr(self, max_ranges=10):
+        """
+        Creates a string representation of the selected branches that can be used as a readable
+        description or postfix in output paths. When the branches of this workflow are configured
+        via the *branches* parameter, and there are more than *max_ranges* identified ranges, the
+        string will contain a unique hash describing those ranges.
+        """
+        self.get_branch_map()
+        if self.branches:
+            ranges = range_join(self.branches)
+            if len(ranges) > max_ranges:
+                return "{}_ranges_{}".format(len(ranges), create_hash(ranges))
+            else:
+                return "_".join(("{}" if len(r) == 1 else "{}To{}").format(*r) for r in ranges)
         else:
-            if self._branch_tasks is None:
-                branch_map = self.get_branch_map()
-                if branch_map is None:
-                    raise AttributeError("workflow task '{}' requires a branch_map".format(self))
-
-                self._branch_tasks = OrderedDict()
-                for b in branch_map:
-                    self._branch_tasks[b] = self.req(self, branch=b,
-                        _exclude=self.exclude_params_branch)
-
-            return self._branch_tasks
+            return "{}To{}".format(self.start_branch, self.end_branch)
 
     def workflow_requires(self):
         """
@@ -593,9 +706,9 @@ class BaseWorkflow(Task):
         this method is called from a branch task, an exception is raised.
         """
         if self.is_branch():
-            raise Exception("calls to workflow_requires are forbidden for branch tasks")
+            return self.as_workflow().workflow_requires()
 
-        return OrderedDict()
+        return DotDict()
 
     def workflow_input(self):
         """
@@ -620,6 +733,59 @@ class BaseWorkflow(Task):
 
         return self.__class__.requires(self)
 
+    def _handle_scheduler_messages(self):
+        if self.scheduler_messages:
+            while not self.scheduler_messages.empty():
+                msg = self.scheduler_messages.get()
+                self.handle_scheduler_message(msg)
+
+    def handle_scheduler_message(self, msg, _attr_value=None):
+        """ handle_scheduler_message(msg)
+        Hook that is called when a scheduler message *msg* is received. Returns *True* when the
+        messages was handled, and *False* otherwise.
+
+        Handled messages:
+
+            - ``tolerance = <int/float>``
+            - ``acceptance = <int/float>``
+        """
+        attr, value = _attr_value or (None, None)
+
+        # handle "tolerance"
+        if attr is None:
+            m = re.match(r"^\s*(tolerance)\s*(\=|\:)\s*(.*)\s*$", str(msg))
+            if m:
+                attr = "tolerance"
+                try:
+                    self.tolerance = float(m.group(3))
+                    value = self.tolerance
+                except ValueError as e:
+                    value = e
+
+        # handle "acceptance"
+        if attr is None:
+            m = re.match(r"^\s*(acceptance)\s*(\=|\:)\s*(.*)\s*$", str(msg))
+            if m:
+                attr = "acceptance"
+                try:
+                    self.acceptance = float(m.group(3))
+                    value = self.acceptance
+                except ValueError as e:
+                    value = e
+
+        # respond
+        if attr:
+            if isinstance(value, Exception):
+                msg.respond("cannot set {}: {}".format(attr, value))
+                logger.info("cannot set {} of task {}: {}".format(attr, self, value))
+            else:
+                msg.respond("{} set to {}".format(attr, value))
+                logger.info("{} of task {} set to {}".format(attr, self, value))
+            return True
+        else:
+            msg.respond("task cannot handle scheduler message: {}".format(msg))
+            return False
+
 
 BaseWorkflow.workflow_property = workflow_property
 BaseWorkflow.cached_workflow_property = cached_workflow_property
