@@ -10,7 +10,6 @@ __all__ = ["SubmissionData", "StatusData", "BaseRemoteWorkflowProxy", "BaseRemot
 
 import sys
 import time
-import math
 import re
 import random
 import threading
@@ -24,7 +23,7 @@ import six
 from law.workflow.base import BaseWorkflow, BaseWorkflowProxy
 from law.job.dashboard import NoJobDashboard
 from law.parameter import NO_FLOAT, NO_INT, get_param, DurationParameter
-from law.util import is_number, iter_chunks, merge_dicts, ShorthandDict
+from law.util import is_number, iter_chunks, merge_dicts, human_duration, DotDict, ShorthandDict
 
 
 logger = logging.getLogger(__name__)
@@ -53,12 +52,12 @@ class SubmissionData(ShorthandDict):
     dummy_job_id = "dummy_job_id"
 
     @classmethod
-    def job_data(cls, job_id=dummy_job_id, branches=None, **kwargs):
+    def job_data(cls, job_id=dummy_job_id, branches=None, log_file=None, **kwargs):
         """
-        Returns a dictionary containing default job submission information such as the *job_id* and
-        task *branches* covered by the job.
+        Returns a dictionary containing default job submission information such as the *job_id*,
+        task *branches* covered by the job, and an optional *log_file*.
         """
-        return dict(job_id=job_id, branches=branches or [])
+        return dict(job_id=job_id, branches=branches or [], log_file=log_file)
 
     def __len__(self):
         return len(self.jobs) + len(self.unsubmitted_jobs)
@@ -93,7 +92,7 @@ class StatusData(ShorthandDict):
     dummy_job_id = SubmissionData.dummy_job_id
 
     @classmethod
-    def job_data(cls, job_id=dummy_job_id, status=None, code=None, error=None, **kwargs):
+    def job_data(cls, job_id=dummy_job_id, status=None, code=None, error=None):
         """
         Returns a dictionary containing default job status information such as the *job_id*, a job
         *status* string, a job return code, and an *error* message.
@@ -284,12 +283,10 @@ class BaseRemoteWorkflowProxy(BaseWorkflowProxy):
 
     def _can_skip_job(self, job_num, branches):
         """
-        Returns *True* when a job can be potentially skipped, which is the case when
-        :py:attr:`only_missing` is *True* and all branch tasks given by *branches* are complete.
+        Returns *True* when a job can be potentially skipped, which is the case when all branch
+        tasks given by *branches* are complete.
         """
-        if not self.task.only_missing:
-            return False
-        elif job_num in self.skip_jobs:
+        if job_num in self.skip_jobs:
             return self.skip_jobs[job_num]
         else:
             self.skip_jobs[job_num] = all(
@@ -356,12 +353,14 @@ class BaseRemoteWorkflowProxy(BaseWorkflowProxy):
             return super(BaseRemoteWorkflowProxy, self).complete()
 
     def requires(self):
-        reqs = OrderedDict()
-
-        # add upstream and workflow specific requirements when not controlling running jobs
-        if not self.task.is_controlling_remote_jobs():
-            reqs.update(super(BaseRemoteWorkflowProxy, self).requires())
-            reqs.update(self._get_task_attribute("workflow_requires")())
+        # use upstream and workflow specific requirements only when not controlling running jobs
+        if self.task.is_controlling_remote_jobs():
+            reqs = DotDict()
+        else:
+            reqs = super(BaseRemoteWorkflowProxy, self).requires()
+            remote_reqs = self._get_task_attribute("workflow_requires")()
+            if remote_reqs:
+                reqs.update(remote_reqs)
 
         return reqs
 
@@ -378,7 +377,7 @@ class BaseRemoteWorkflowProxy(BaseWorkflowProxy):
         out_dir = self._get_task_attribute("output_directory")()
 
         # define outputs
-        outputs = OrderedDict()
+        outputs = DotDict()
         postfix = self._get_task_attribute("output_postfix")()
 
         # a file containing the submission data, i.e. job ids etc
@@ -414,6 +413,8 @@ class BaseRemoteWorkflowProxy(BaseWorkflowProxy):
         Actual run method that starts the processing of jobs and initiates the status polling, or
         performs job cancelling or cleaning, depending on the task parameters.
         """
+        super(BaseRemoteWorkflowProxy, self).run()
+
         task = self.task
         self._outputs = self.output()
 
@@ -475,9 +476,9 @@ class BaseRemoteWorkflowProxy(BaseWorkflowProxy):
                     self.submit()
 
                     # sleep once to give the job interface time to register the jobs
-                    post_submit_delay = self._get_task_attribute("post_submit_delay")()
-                    if post_submit_delay:
-                        logger.debug("sleep for {} seconds due to post_submit_delay".format(
+                    post_submit_delay = self._get_task_attribute("post_submit_delay", True)()
+                    if post_submit_delay > 0:
+                        logger.debug("sleep for {} second(s) due to post_submit_delay".format(
                             post_submit_delay))
                         time.sleep(post_submit_delay)
 
@@ -576,49 +577,55 @@ class BaseRemoteWorkflowProxy(BaseWorkflowProxy):
         # collect data of jobs that should be submitted: num -> branches
         submit_jobs = OrderedDict()
 
+        # keep track of the list of unsubmitted job nums before retry jobs are handled to control
+        # whether they are resubmitted immediately or at the end (subject to shuffling)
+        unsubmitted_job_nums = list(self.submission_data.unsubmitted_jobs.keys())
+        if task.shuffle_jobs:
+            random.shuffle(unsubmitted_job_nums)
+
         # handle jobs for resubmission
         if retry_jobs:
             for job_num, branches in six.iteritems(retry_jobs):
-                # some retry jobs can be skipped
+                # some retry jobs might be skipped
                 if self._can_skip_job(job_num, branches):
                     continue
 
-                # the number of parallel jobs might be reached as well
-                # in that case, add the jobs back to the unsubmitted ones and update the job id
+                # in the case that the number of parallel jobs might be reached, or when retry jobs
+                # are configured to be tried last, add the jobs back to the unsubmitted ones and
+                # update the job id
                 n = self.poll_data.n_active + len(submit_jobs)
-                if n >= self.poll_data.n_parallel:
+                if n >= self.poll_data.n_parallel or task.append_retry_jobs:
+                    self.submission_data.jobs.pop(job_num, None)
                     self.submission_data.unsubmitted_jobs[job_num] = branches
-                    del self.submission_data.jobs[job_num]
+                    if task.append_retry_jobs:
+                        unsubmitted_job_nums.append(job_num)
+                    else:
+                        unsubmitted_job_nums.insert(0, job_num)
                     continue
 
                 # mark job for resubmission
                 submit_jobs[job_num] = sorted(branches)
 
-        # fill with jobs from the list of unsubmitted jobs until maximum number of parallel jobs is
-        # reached
-        new_jobs = OrderedDict()
-        for job_num, branches in list(self.submission_data.unsubmitted_jobs.items()):
+        # fill with unsubmitted jobs until maximum number of parallel jobs is reached
+        for job_num in unsubmitted_job_nums:
+            branches = self.submission_data.unsubmitted_jobs[job_num]
+
             # remove jobs that don't need to be submitted
             if self._can_skip_job(job_num, branches):
-                del self.submission_data.unsubmitted_jobs[job_num]
+                self.submission_data.unsubmitted_jobs.pop(job_num, None)
                 continue
 
             # do nothing when n_parallel is already reached
-            n = self.poll_data.n_active + len(submit_jobs) + len(new_jobs)
+            n = self.poll_data.n_active + len(submit_jobs)
             if n >= self.poll_data.n_parallel:
                 continue
 
             # mark jobs for submission
-            del self.submission_data.unsubmitted_jobs[job_num]
-            new_jobs[job_num] = sorted(branches)
+            self.submission_data.unsubmitted_jobs.pop(job_num, None)
+            submit_jobs[job_num] = sorted(branches)
 
-        # add new jobs to the jobs to submit, maybe also shuffle
+        # store submission data for jobs about to be submitted
         new_submission_data = OrderedDict()
-        new_job_nums = list(new_jobs.keys())
-        if task.shuffle_jobs:
-            random.shuffle(new_job_nums)
-        for job_num in new_job_nums:
-            submit_jobs[job_num] = new_jobs[job_num]
 
         # when there is nothing to submit, dump the submission data to the output file and stop here
         if not submit_jobs:
@@ -631,8 +638,10 @@ class BaseRemoteWorkflowProxy(BaseWorkflowProxy):
             job_data = self.submission_data_cls.job_data(branches=branches)
             self.submission_data.jobs[job_num] = job_data
 
-        # create job submission files
-        job_files = [self.create_job_file(*tpl) for tpl in six.iteritems(submit_jobs)]
+        # create job submission files mapped to job nums
+        job_files = OrderedDict()
+        for job_num, branches in six.iteritems(submit_jobs):
+            job_files[job_num] = self.create_job_file(job_num, branches)
 
         # log some stats
         dst_info = self.destination_info() or ""
@@ -641,11 +650,11 @@ class BaseRemoteWorkflowProxy(BaseWorkflowProxy):
             len(submit_jobs), self.workflow_type, dst_info))
 
         # actual submission
-        job_ids = self.submit_jobs(job_files)
+        job_ids = self._submit(job_files)
 
         # store submission data
         errors = []
-        for job_num, job_id in six.moves.zip(submit_jobs, job_ids):
+        for job_num, job_id, files in six.moves.zip(submit_jobs, job_ids, job_files.values()):
             # handle errors
             if isinstance(job_id, Exception):
                 errors.append((job_num, job_id))
@@ -654,7 +663,8 @@ class BaseRemoteWorkflowProxy(BaseWorkflowProxy):
             # set the job id in the job data
             job_data = self.submission_data.jobs[job_num]
             job_data["job_id"] = job_id
-            new_submission_data[job_num] = job_data
+            job_data["log_file"] = files.get("log")
+            new_submission_data[job_num] = job_data.copy()
 
             # inform the dashboard
             task.forward_dashboard_event(self.dashboard, job_data, "action.submit", job_num)
@@ -679,38 +689,43 @@ class BaseRemoteWorkflowProxy(BaseWorkflowProxy):
 
         return new_submission_data
 
-    def submit_jobs(self, job_files, **kwargs):
+    def _submit(self, job_files, **kwargs):
         task = self.task
 
+        # job_files is an ordered mapping job_num -> {"job": PATH, "log": PATH/None}, get keys and
+        # values for faster lookup by numeric index
+        job_nums = list(job_files.keys())
+        job_files = [f["job"] for f in six.itervalues(job_files)]
+
         # prepare objects for dumping intermediate submission data
-        dump_freq = self._get_task_attribute("dump_intermediate_submission_data")()
+        dump_freq = self._get_task_attribute("dump_intermediate_submission_data", True)()
         if dump_freq and not is_number(dump_freq):
             dump_freq = 50
 
+        # get job kwargs for submission and merge with passed kwargs
+        submit_kwargs = self._get_job_kwargs("submit")
+        submit_kwargs = merge_dicts(submit_kwargs, kwargs)
+
         # progress callback to inform the scheduler
         def progress_callback(i, job_id):
-            job_num = i + 1
+            job_num = job_nums[i]
 
             # some job managers respond with a list of job ids per submission (e.g. htcondor, slurm)
-            # batched submission is not yet supported, so get the first id
-            if isinstance(job_id, list):
+            # so get the first id as long as batched submission is not yet supported
+            if isinstance(job_id, list) and not self.job_manager.chunk_size_submit:
                 job_id = job_id[0]
 
             # set the job id early
             self.submission_data.jobs[job_num]["job_id"] = job_id
 
             # log a message every 25 jobs
-            if job_num in (1, len(job_files)) or job_num % 25 == 0:
-                task.publish_message("submitted {}/{} job(s)".format(job_num, len(job_files)))
+            if i in (0, len(job_files) - 1) or (i + 1) % 25 == 0:
+                task.publish_message("submitted {}/{} job(s)".format(i + 1, len(job_files)))
 
             # dump intermediate submission data with a certain frequency
-            if dump_freq and job_num % dump_freq == 0:
+            if dump_freq and (i + 1) % dump_freq == 0:
                 self.dump_submission_data()
 
-        # get job kwargs for submission and merge with passed kwargs
-        submit_kwargs = self._get_job_kwargs("submit")
-        submit_kwargs = merge_dicts(submit_kwargs, kwargs)
-
         return self.job_manager.submit_batch(job_files, retries=3, threads=task.threads,
             callback=progress_callback, **submit_kwargs)
 
@@ -723,35 +738,36 @@ class BaseRemoteWorkflowProxy(BaseWorkflowProxy):
         # total job count
         n_jobs = len(self.submission_data)
 
-        # get job kwargs for status querying
-        query_kwargs = self._get_job_kwargs("query")
-
-        # store the number of consecutive polling failures and get the maximum number of polls
-        n_poll_fails = 0
-        if task.walltime == NO_FLOAT:
-            max_polls = sys.maxsize
-        else:
-            max_polls = int(math.ceil((task.walltime * 3600.) / (task.poll_interval * 60.)))
-
-        # update variable attributes for polling
-        self.poll_data.n_finished_min = task.acceptance * (1 if task.acceptance > 1 else n_jobs)
-        self.poll_data.n_failed_max = task.tolerance * (1 if task.tolerance > 1 else n_jobs)
-
         # track finished and failed jobs in dicts holding status data
         finished_jobs = OrderedDict()
         failed_jobs = OrderedDict()
 
+        # track number of consecutive polling failures and the start time
+        n_poll_fails = 0
+        start_time = time.time()
+
+        # get job kwargs for status querying
+        query_kwargs = self._get_job_kwargs("query")
+
         # start the poll loop
-        for i in six.moves.range(max_polls):
+        i = -1
+        while True:
+            i += 1
+
             # sleep after the first iteration
             if i > 0:
                 time.sleep(task.poll_interval * 60)
 
-            # handle scheduler messages first
-            if task.scheduler_messages:
-                while not task.scheduler_messages.empty():
-                    msg = task.scheduler_messages.get()
-                    task.handle_scheduler_message(msg)
+            # handle scheduler messages, which could change task some parameters
+            task._handle_scheduler_messages()
+
+            # walltime exceeded?
+            if task.walltime != NO_FLOAT and (time.time() - start_time) > task.walltime * 3600:
+                raise Exception("exceeded walltime: {}".format(human_duration(hours=task.walltime)))
+
+            # update variable attributes for polling
+            self.poll_data.n_finished_min = task.acceptance * (1 if task.acceptance > 1 else n_jobs)
+            self.poll_data.n_failed_max = task.tolerance * (1 if task.tolerance > 1 else n_jobs)
 
             # determine the currently active jobs, i.e., the jobs whose states should be checked,
             # and also store jobs whose ids are unknown
@@ -835,6 +851,7 @@ class BaseRemoteWorkflowProxy(BaseWorkflowProxy):
                 elif data["status"] == self.job_manager.FINISHED:
                     finished_jobs[job_num] = data
                     self.poll_data.n_active -= 1
+                    self.submission_data.jobs[job_num]["job_id"] = self.submission_data.dummy_job_id
                     task.forward_dashboard_event(self.dashboard, data, "status.finished", job_num)
 
                 elif data["status"] in (self.job_manager.FAILED, self.job_manager.RETRY):
@@ -880,15 +897,22 @@ class BaseRemoteWorkflowProxy(BaseWorkflowProxy):
             # log newly failed jobs
             if newly_failed_jobs:
                 print("{} failed job(s) in task {}:".format(len(newly_failed_jobs), task.task_id))
-                tmpl = "    job: {}, branch(es): {}, id: {job_id}, status: {status}, " \
-                    "code: {code}, error: {error}{}"
+                tmpl = "    job: {job_num}, branch(es): {branches}, id: {job_id}, " \
+                    "status: {status}, code: {code}, error: {error}{ext}"
+
                 for i, (job_num, data) in enumerate(six.iteritems(newly_failed_jobs)):
                     branches = self.submission_data.jobs[job_num]["branches"]
-                    law_err = ""
+                    log_file = self.submission_data.jobs[job_num]["log_file"]
+                    ext = ""
                     if data["code"] in self.job_error_messages:
                         law_err = self.job_error_messages[data["code"]]
-                        law_err = ", job script error: {}".format(law_err)
-                    print(tmpl.format(job_num, ",".join(str(b) for b in branches), law_err, **data))
+                        ext += ", job script error: {}".format(law_err)
+                    if log_file:
+                        ext += ", log: {}".format(log_file)
+
+                    print(tmpl.format(job_num=job_num, branches=",".join(str(b) for b in branches),
+                        ext=ext, **data))
+
                     if i + 1 >= self.show_errors:
                         remaining = len(newly_failed_jobs) - self.show_errors
                         if remaining > 0:
@@ -923,17 +947,18 @@ class BaseRemoteWorkflowProxy(BaseWorkflowProxy):
             # configurable poll callback
             task.poll_callback(self.poll_data)
 
-            # trigger automatic resubmission and submission of unsubmitted jobs
-            self.submit(retry_jobs)
+            # trigger automatic resubmission and submission of unsubmitted jobs if necessary
+            if retry_jobs or self.poll_data.n_active < self.poll_data.n_parallel:
+                self.submit(retry_jobs)
 
             # break when no polling is desired
             # we can get to this point when there was already a submission and the no_poll
             # parameter was set so that only failed jobs are resubmitted once
             if task.no_poll:
                 break
-        else:
-            # walltime exceeded
-            raise Exception("walltime exceeded")
+
+        duration = round(time.time() - start_time)
+        task.publish_message("polling took {}".format(human_duration(seconds=duration)))
 
 
 class BaseRemoteWorkflow(BaseWorkflow):
@@ -959,6 +984,13 @@ class BaseRemoteWorkflow(BaseWorkflow):
        Alignment value that is passed to :py:meth:`law.job.base.BaseJobManager.status_line` to print
        the status line during job status polling. Defaults to *False*.
 
+    .. py:classattribute:: append_retry_jobs
+       type: bool
+
+       When *True*, jobs to retry are added to the end of the jobs to submit, giving priority to new
+       ones. However, when *shuffle_jobs* is *True*, they might be submitted again earlier.
+       Defaults to *False*.
+
     .. py:classattribute:: retries
        type: luigi.IntParameter
 
@@ -976,11 +1008,6 @@ class BaseRemoteWorkflow(BaseWorkflow):
        Maximum number of parallel running jobs, e.g. to protect a very busy queue of a batch system.
        Empty default value (infinity).
 
-    .. py:classattribute:: only_missing
-       type: luigi.BoolParameter
-
-       When *True*, only consider incomplete tasks for job submisson. Defaults to *False*.
-
     .. py:classattribute:: no_poll
        type: luigi.BoolParameter
 
@@ -1035,42 +1062,49 @@ class BaseRemoteWorkflow(BaseWorkflow):
     """
 
     retries = luigi.IntParameter(default=5, significant=False, description="number of automatic "
-        "resubmission attempts per job, default: 5")
+        "resubmission attempts per job; default: 5")
     tasks_per_job = luigi.IntParameter(default=1, significant=False, description="number of tasks "
-        "to be processed by one job, default: 1")
+        "to be processed by one job; default: 1")
     parallel_jobs = luigi.IntParameter(default=NO_INT, significant=False, description="maximum "
-        "number of parallel running jobs, default: infinite")
-    only_missing = luigi.BoolParameter(significant=False, description="skip tasks that are "
-        "considered complete")
-    no_poll = luigi.BoolParameter(significant=False, description="just submit, do not initiate "
-        "status polling after submission")
+        "number of parallel running jobs; default: infinite")
+    no_poll = luigi.BoolParameter(default=False, significant=False, description="just submit, do "
+        "not initiate status polling after submission; default: False")
     threads = luigi.IntParameter(default=4, significant=False, description="number of threads to "
-        "use for (re)submission and status queries, default: 4")
+        "use for (re)submission and status queries; default: 4")
     walltime = DurationParameter(default=NO_FLOAT, unit="h", significant=False,
-        description="maximum wall time, default unit is hours, default: not set")
+        description="maximum wall time; default unit is hours; default: infinite")
     poll_interval = DurationParameter(default=1, unit="m", significant=False, description="time "
-        "between status polls, default unit is minutes, default: 1")
+        "between status polls; default unit is minutes; default: 1")
     poll_fails = luigi.IntParameter(default=5, significant=False, description="maximum number of "
-        "consecutive errors during polling, default: 5")
-    shuffle_jobs = luigi.BoolParameter(description="shuffled job submission", significant=False)
-    cancel_jobs = luigi.BoolParameter(description="cancel all submitted jobs, no new submission")
-    cleanup_jobs = luigi.BoolParameter(description="cleanup all submitted jobs, no new submission")
-    ignore_submission = luigi.BoolParameter(significant=False, description="ignore any existing "
-        "submission file from a previous submission and start a new one")
-    transfer_logs = luigi.BoolParameter(significant=False, description="transfer job logs to the "
-        "output directory")
+        "consecutive errors during polling; default: 5")
+    shuffle_jobs = luigi.BoolParameter(default=False, significant=False, description="shuffled job "
+        "submission; default: False")
+    cancel_jobs = luigi.BoolParameter(default=False, description="cancel all submitted jobs but do "
+        "not submit new jobs; default: False")
+    cleanup_jobs = luigi.BoolParameter(default=False, description="cleanup all submitted jobs but "
+        "do not submit new jobs; default: False")
+    ignore_submission = luigi.BoolParameter(default=False, significant=False, description="ignore "
+        "any existing submission file from a previous submission and start a new one; default: "
+        "False")
+    transfer_logs = luigi.BoolParameter(default=False, significant=False, description="transfer "
+        "job logs to the output directory; default: False")
 
-    align_polling_status_line = False
     check_unreachable_acceptance = False
+    align_polling_status_line = False
+    append_retry_jobs = False
+
+    exclude_index = True
 
     exclude_params_branch = {
-        "retries", "tasks_per_job", "parallel_jobs", "only_missing", "no_poll", "threads",
-        "walltime", "poll_interval", "poll_fails", "shuffle_jobs", "cancel_jobs", "cleanup_jobs",
+        "retries", "tasks_per_job", "parallel_jobs", "no_poll", "threads", "walltime",
+        "poll_interval", "poll_fails", "shuffle_jobs", "cancel_jobs", "cleanup_jobs",
         "ignore_submission", "transfer_logs",
     }
     exclude_params_repr = {"cancel_jobs", "cleanup_jobs"}
 
-    exclude_index = True
+    abstract_workflow_methods = BaseWorkflow.abstract_workflow_methods | {
+        "workflow_requires", "output_directory", "output_postfix",
+    }
 
     def inst_exclude_params_repr(self):
         params = super(BaseRemoteWorkflow, self).inst_exclude_params_repr()
@@ -1092,6 +1126,16 @@ class BaseRemoteWorkflow(BaseWorkflow):
         """
         return
 
+    def post_submit_delay(self):
+        """
+        Configurable delay in seconds to wait after submitting jobs and before starting the status
+        polling.
+        """
+        return self.poll_interval * 60
+
+    def dump_intermediate_submission_data(self):
+        return True
+
     def create_job_dashboard(self):
         """
         Hook method to return a configured :py:class:`law.job.BaseJobDashboard` instance that will
@@ -1128,21 +1172,79 @@ class BaseRemoteWorkflow(BaseWorkflow):
 
         return isinstance(self.workflow_proxy, BaseRemoteWorkflowProxy)
 
-    def handle_scheduler_message(self, msg):
-        """
-        Hook that is called when a scheduler message *msg* is received during polling. By default,
-        this method checks for messages with the format ``"parallel_jobs:<int>"`` to dynamically set
-        the number of parallely running jobs.
-        """
-        if not getattr(self, "workflow_proxy", None):
-            return
+    def handle_scheduler_message(self, msg, _attr_value=None):
+        """ handle_scheduler_message(msg)
+        Hook that is called when a scheduler message *msg* is received. Returns *True* when the
+        messages was handled, and *False* otherwise.
 
-        m = re.match(r"^\s*(parallel\_jobs)\s*(\=|\:)\s*((|\+|\-)\d+)\s*$", str(msg))
-        if m:
-            n = int(m.group(3))
-            n = self.workflow_proxy._set_parallel_jobs(n)
-            n_str = str(n) if n != self.workflow_proxy.n_parallel_max else "unlimited"
-            msg.respond("number of parallel jobs set to {}".format(n_str))
-            logger.info("number of parallel jobs of task {} set to {}".format(self, n_str))
-        else:
-            msg.respond("task cannot handle scheduler message: {}".format(msg))
+        Handled messages in addition to those defined in
+        :py:meth:`law.workflow.base.BaseWorkflow.handle_scheduler_message`:
+
+            - ``parallel_jobs = <int>``
+            - ``walltime = <str/int/float>``
+            - ``poll_fails = <int>``
+            - ``poll_interval = <str/int/float>``
+            - ``retries = <int>``
+        """
+        attr, value = _attr_value or (None, None)
+
+        # handle "parallel_jobs"
+        if attr is None:
+            m = re.match(r"^\s*(parallel\_jobs)\s*(\=|\:)\s*(.*)\s*$", str(msg))
+            if m:
+                attr = "parallel_jobs"
+                # the workflow proxy must be set here
+                if not getattr(self, "workflow_proxy", None):
+                    value = Exception("workflow_proxy not set yet")
+                else:
+                    try:
+                        n = self.workflow_proxy._set_parallel_jobs(int(m.group(3)))
+                        value = "unlimited" if n == self.workflow_proxy.n_parallel_max else str(n)
+                    except ValueError as e:
+                        value = e
+
+        # handle "walltime"
+        if attr is None:
+            m = re.match(r"^\s*(walltime)\s*(\=|\:)\s*(.*)\s*$", str(msg))
+            if m:
+                attr = "walltime"
+                try:
+                    self.walltime = self.__class__.walltime.parse(m.group(3))
+                    value = human_duration(hours=self.walltime, colon_format=True)
+                except ValueError as e:
+                    value = e
+
+        # handle "poll_fails"
+        if attr is None:
+            m = re.match(r"^\s*(poll\_fails)\s*(\=|\:)\s*(.*)\s*$", str(msg))
+            if m:
+                attr = "poll_fails"
+                try:
+                    self.poll_fails = int(m.group(3))
+                    value = self.poll_fails
+                except ValueError as e:
+                    value = e
+
+        # handle "poll_interval"
+        if attr is None:
+            m = re.match(r"^\s*(poll\_interval)\s*(\=|\:)\s*(.*)\s*$", str(msg))
+            if m:
+                attr = "poll_interval"
+                try:
+                    self.poll_interval = self.__class__.poll_interval.parse(m.group(3))
+                    value = human_duration(minutes=self.poll_interval, colon_format=True)
+                except ValueError as e:
+                    value = e
+
+        # handle "retries"
+        if attr is None:
+            m = re.match(r"^\s*(retries)\s*(\=|\:)\s*(.*)\s*$", str(msg))
+            if m:
+                attr = "retries"
+                try:
+                    self.retries = int(m.group(3))
+                    value = self.retries
+                except ValueError as e:
+                    value = e
+
+        return super(BaseRemoteWorkflow, self).handle_scheduler_message(msg, (attr, value))
